# Antragsraum-Proxy

Der Antragsraum-Proxy nimmt HTTP Requests vom Antragsraum-Server entgegen, mappt diese auf gRPC
und leitet sie an einen Vorgang-Manager weiter. Zum Protokollmapping von HTTP auf gRPC wird die
Bibliothek [gRPC-Gateway](https://grpc-ecosystem.github.io/grpc-gateway/) verwendet. Dadurch können die HTTP-Endpunkte inkl. des Mappings 
größtenteils automatisch aus proto-Dateien generiert werden.

Requests werden zunächst an den gRPC-Router weitergeleitet. Das ist ein Proxy-eigener 
gRPC-Server, der auf localhost und dem unter config.grpc.router.port eingetragenen Port läuft. 
Dort wird die Adresse des Ziel-Vorgang-Managers aus einem HTTP Header extrahiert. Der Port des
Ziel-Vorgang-Managers ist konstant und wird unter config.grpc.server.port festgelegt. 
Anschließend leitet der gRPC-Router die gRPC Request an die URL weiter, die aus der 
extrahierten Adresse und dem Port zusammengesetzt wird.

## Getting Started

### Dependencies installieren

```
go mod download
```

```
go install \
    github.com/grpc-ecosystem/grpc-gateway/v2/protoc-gen-grpc-gateway \
    github.com/grpc-ecosystem/grpc-gateway/v2/protoc-gen-openapiv2 \
    google.golang.org/protobuf/cmd/protoc-gen-go \
    google.golang.org/grpc/cmd/protoc-gen-go-grpc \
    github.com/sirupsen/logrus \
    go.elastic.co/ecslogrus
```

### Buf installieren

https://github.com/bufbuild/buf?tab=readme-ov-file

### Proto Dateien anpassen

Die *.proto Dateien, die die verwendeten GRPC Apis des Proxys definieren müssen aus den entsprechenden Projekten
in das Verzeichnis api/proto kopiert werden.

Die Projekte sind:

* [VorgangManager](https://code.schleswig-holstein.de/ozg-cloud/app/vorgang-manager/-/tree/main/vorgang-manager-interface/src/main/protobuf)
* [NachrichtenManager](https://code.schleswig-holstein.de/ozg-cloud/app/nachrichten-manager/-/tree/main/nachrichten-manager-interface/src/main/protobuf)

Damit die Code generierung funktioniert müssen die proto-Dateien aktuell noch angepasst werden.
Dazu muss in jede Datei die Zeile ```option go_package = "de.ozgcloud.antragraumproxy";``` hinzugefügt werden, damit
die erzeugten Quellcode-Dateien im selben go package liegen.

### Generierung konfigurieren

Die Generierung des Gateways kann mit den Dateien

* api/gateway-config.yml - go Code für den Gateway
* api/openapi-config.yml - für die Erzeugung der swagger json Dateien

konfiguriert werden.

### Protobuf-Code und swagger Datei generieren

```
buf generate
```

Erzeugt dann den Code für das Gateway und auch die Datei gen/openapiv2/openapiv2.swagger.json.
Diese Datei kann dann in anderen Projekten verwendet werden um die nötige DTO Klassen zu erzeugen.

### Anwendung starten

```
ACTIVE_PROFILE=local go run cmd/antragsraum-proxy/main.go
```

## Config

config.yml is the default config for deployment

[Config-Datei](./config/config.yml)

config-local.yml is the config file for local development
setting ACTIVE_PROFILE=local will load the config-local.yml

[Config-Datei](./config/config-local.yml)

```
http:
  server:
    port: Port des HTTP Gateways (int)
grpc:
  server:
    mock: gRPC Server mocken (bool)
    port: Port des gRPC Servers (int)
  router:
    port: Port des gRPC Routers (int)
logging:
  level: "ERROR" | "WARN" | "INFO" | "DEBUG"
```

### Releasing

Diese Schritte ausführen:

* Master-Branch auschecken
* Master-Branch updaten
* Release-Branch auschecken
* Release-Branch updaten
* Master in den Release-Branch mergen - noch nicht pushen!
* Die Version von 'var version' in cmd/antragraum-proxy/main.go "vX.X.X-beta.x" auf "vX.X.X" ändern
* Commit erstellen: "release version X.X.X"
* Release Branch pushen
* Tag "X.X.X" erstellen: git tag "X.X.X"
* tag pushen: git push origin X.X.X
* Master auschecken
* Release-Branch in den Master mergen nicht pushen!
* Die Version von 'var version' in cmd/antragraum-proxy/main.go "vX.X.X" auf "vX.X.X-beta.x" ändern
* Master Branch pushen
