//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package config

import (
	"github.com/stretchr/testify/assert"
	"os"
	"testing"
)

func TestLoadConfig(t *testing.T) {
	t.Run("should load default config", func(t *testing.T) {
		config := LoadConfig()

		expectedConfig := Config{}
		expectedConfig.Grpc.Server.Mock = false
		expectedConfig.Http.Server.Port = 8082
		expectedConfig.Grpc.Server.Port = 50052
		expectedConfig.Grpc.Router.Port = 50051
		expectedConfig.Logging.Level = "DEBUG"

		assert.Equal(t, expectedConfig, config)
	})

	t.Run("should load local config", func(t *testing.T) {
		assert.NoError(t, os.Setenv(activeProfileParam, "local"), "Setenv "+activeProfileParam+" should not return an error")

		config := LoadConfig()

		assert.NoError(t, os.Unsetenv(activeProfileParam), "Unsetenv "+activeProfileParam+" should not return an error")

		expectedConfig := Config{}
		expectedConfig.Grpc.Server.Mock = true
		expectedConfig.Http.Server.Port = 8082
		expectedConfig.Grpc.Server.Port = 50052
		expectedConfig.Grpc.Router.Port = 50051
		expectedConfig.Logging.Level = "INFO"

		assert.Equal(t, expectedConfig, config)
	})
}
