//
// Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
// Ministerpräsidenten des Landes Schleswig-Holstein
// Staatskanzlei
// Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
//
// Lizenziert unter der EUPL, Version 1.2 oder - sobald
// diese von der Europäischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie dürfen dieses Werk ausschließlich gemäß
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
//
// https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
//
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
// ausdrücklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschränkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package server

import (
	"context"
	"fmt"
	"net/http"
    log "github.com/sirupsen/logrus"
	"github.com/grpc-ecosystem/grpc-gateway/v2/runtime"
	"google.golang.org/grpc/metadata"
)

func HeaderMatcher(header string) (string, bool) {
	if header == GrpcAddressHeader {
		return header, true
	}

	return header, false
}

func StartHttpGateway() *http.Server {
	md := metadata.Pairs("CLIENT_NAME-bin", "antragraum-proxy")
	ctx := metadata.NewOutgoingContext(context.Background(), md)
	ctx, cancel := context.WithCancel(ctx)
	defer cancel()

	mux := runtime.NewServeMux(runtime.WithErrorHandler(ErrorHandler), runtime.WithIncomingHeaderMatcher(HeaderMatcher))
	grpcRouterUrl := fmt.Sprintf("localhost:%d", conf.Grpc.Router.Port)

	RegisterHomeEndpoint(mux)
	RegisterAntragraumEndpoints(ctx, mux, grpcRouterUrl)
	RegisterCommandEndpoints(ctx, mux, grpcRouterUrl)
	RegisterUploadAttachmentEndpoint(mux)
	RegisterGetAttachmentContentEndpoint(mux)

	httpServer := &http.Server{
		Addr:    fmt.Sprintf(":%d", conf.Http.Server.Port),
		Handler: RequestLoggingMiddleware(mux),
	}

	log.Info(fmt.Sprintf("HTTP gateway listening on port %d", conf.Http.Server.Port))
	if err := httpServer.ListenAndServe(); err != nil {
		log.Fatal(fmt.Sprintf("HTTP gateway failed to serve: %v", err))
	}

	return httpServer
}
